@@ -24,11 +24,11 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
+
 #include "gdkx.h"
 #include "gdkregion-generic.h"
 
-#include <config.h>
-
 #include <pango/pangoxft.h>
 
 #include <stdlib.h>
@@ -312,22 +312,29 @@ gdk_x11_drawable_get_xft_draw (GdkDrawable *drawable)
    if (impl->xft_draw == NULL)
     {
       GdkColormap *colormap = gdk_drawable_get_colormap (drawable);
-      GdkVisual *visual;
-
-      if (!colormap)
+      
+      if (colormap)
 	{
+          GdkVisual *visual;
+
+          visual = gdk_colormap_get_visual (colormap);
+      
+          impl->xft_draw = XftDrawCreate (GDK_SCREEN_XDISPLAY (impl->screen), impl->xid,
+ 				          GDK_VISUAL_XVISUAL (visual), GDK_COLORMAP_XCOLORMAP (colormap));
+	}
+      else if (gdk_drawable_get_depth (drawable) == 1)
+	{
+	  impl->xft_draw = XftDrawCreateBitmap (GDK_SCREEN_XDISPLAY (impl->screen), impl->xid);
+	}
+      else
+        {
 	  g_warning ("Using Xft rendering requires the drawable argument to\n"
 		     "have a specified colormap. All windows have a colormap,\n"
 		     "however, pixmaps only have colormap by default if they\n"
 		     "were created with a non-NULL window argument. Otherwise\n"
 		     "a colormap must be set on them with gdk_drawable_set_colormap");
- 	  return NULL;
-	}
-
-      visual = gdk_colormap_get_visual (colormap);
-      
-      impl->xft_draw = XftDrawCreate (GDK_SCREEN_XDISPLAY (impl->screen), impl->xid,
- 				      GDK_VISUAL_XVISUAL (visual), GDK_COLORMAP_XCOLORMAP (colormap));
+	  return NULL;
+        }
     }
 
    return impl->xft_draw;
@@ -352,6 +359,7 @@ gdk_x11_drawable_update_xft_clip (GdkDrawable *drawable,
     {
       GdkRegionBox *boxes = gc_private->clip_region->rects;
       gint n_boxes = gc_private->clip_region->numRects;
+#if 0				/* Until XftDrawSetClipRectangles is there */
       XRectangle *rects = g_new (XRectangle, n_boxes);
       int i;
 
@@ -365,6 +373,25 @@ gdk_x11_drawable_update_xft_clip (GdkDrawable *drawable,
       XftDrawSetClipRectangles (xft_draw, 0, 0, rects, n_boxes);
 
       g_free (rects);
+#else
+      Region xregion = XCreateRegion ();
+      int i;
+ 
+      for (i=0; i < n_boxes; i++)
+ 	{
+ 	  XRectangle rect;
+ 	  
+ 	  rect.x = CLAMP (boxes[i].x1 + gc->clip_x_origin, G_MINSHORT, G_MAXSHORT);
+ 	  rect.y = CLAMP (boxes[i].y1 + gc->clip_y_origin, G_MINSHORT, G_MAXSHORT);
+ 	  rect.width = CLAMP (boxes[i].x2 + gc->clip_x_origin, G_MINSHORT, G_MAXSHORT) - rect.x;
+ 	  rect.height = CLAMP (boxes[i].y2 + gc->clip_y_origin, G_MINSHORT, G_MAXSHORT) - rect.y;
+	  
+ 	  XUnionRectWithRegion (&rect, xregion, xregion);
+ 	}
+      
+      XftDrawSetClip (xft_draw, xregion);
+      XDestroyRegion (xregion);
+#endif      
     }
   else
     {
@@ -1233,7 +1260,7 @@ draw_with_images (GdkDrawable       *drawable,
 
   dest_pict = gdk_x11_drawable_get_picture (drawable);  
   
-  pix_gc = gdk_gc_new (pix);
+  pix_gc = _gdk_drawable_get_scratch_gc (pix, FALSE);
 
   for (y0 = 0; y0 < height; y0 += GDK_SCRATCH_IMAGE_HEIGHT)
     {
@@ -1264,7 +1291,6 @@ draw_with_images (GdkDrawable       *drawable,
     XRenderFreePicture (xdisplay, mask);
   
   g_object_unref (pix);
-  g_object_unref (pix_gc);
 }
 
 typedef struct _ShmPixmapInfo ShmPixmapInfo;
@@ -1401,6 +1427,7 @@ gdk_x11_draw_pixbuf (GdkDrawable     *drawable,
 
   if (format_type == FORMAT_NONE ||
       !gdk_pixbuf_get_has_alpha (pixbuf) ||
+      gdk_drawable_get_depth (drawable) == 1 ||
       (dither == GDK_RGB_DITHER_MAX && gdk_drawable_get_depth (drawable) != 24) ||
       gdk_x11_drawable_get_picture (drawable) == None)
     {
