@@ -24,6 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
@@ -1737,7 +1738,8 @@ render_fallback_image (GtkStyle          *style,
  * @style: a #GtkStyle associated with @widget, or %NULL
  * @direction: text direction
  * @state: widget state
- * @size: icon size
+ * @size: icon size. A size of (GtkIconSize)-1
+ *        means render at the size of the source and don't scale.
  * @widget: widget that will display the icon, or %NULL.
  *          The only use that is typically made of this
  *          is to determine the appropriate #GdkScreen.
@@ -2702,7 +2704,10 @@ copy_cache (GtkIconSet *icon_set,
       *icon_copy = *icon;
 
       if (icon_copy->style)
-        attach_to_style (copy_recipient, icon_copy->style);
+	{
+	  attach_to_style (copy_recipient, icon_copy->style);
+	  g_object_ref (icon_copy->style);
+	}
         
       g_object_ref (icon_copy->pixbuf);
 
